dojo.provide( "com.ibm.data.EnablerTreeModel" );
dojo.require("com.ibm.data.DataTreeModel");

// This module provides the API for a data model that corresponds to any data set that contains
// parent-child relationships that can be represented as a tree.  It's exactly the same as the
// model API that's used by dijit.Tree with some functions already implemented for ease of use.
// As a result, any implementation of this API can be used as the model to a dijit.Tree widget
// or any other widget that uses a data model as its backend service provider.  This data API
// also adds some additional constraints and extension points that implementations should support
// in addition to the model API that's used by dijit.Tree, but by no means breaks the API or
// deviates from the required functions of the dijit model API.  One such addition is the
// support for additional query parameters that can be specified when instantiating a model
// that are always appended to any request to the server for data.  This allows particular instances
// of a data model to include metadata information that would not otherwise be included by certain
// data services that support it.
dojo.declare( "com.ibm.data.EnablerTreeModel",
			com.ibm.data.DataTreeModel,
			{
				// model: The Enabler TreeModel implementation
				eModel: null,
				
				// store: The dojo.data.Read/Write implementation 
				store: null,
				
				// query: The query used to retrieve the root item for this model
				query: null,
				
				// root: The root item of this model
				root: null,
				
				// addQueryParams: Object additional query parameters that should be appended
				//		to any request for items from the back end.  Is useful for getting
				// 		additional information such as metadata for each item from a data service 
				// 		that provides it.
				addQueryParams: null,
				
				constructor: function(args) {
					dojo.mixin(this, args);
					
					if(!this.addQueryParams) this.addQueryParams = {};
					
					this.connections = [];
					if(this.store.getFeatures()["dojo.data.api.Notification"]) {
						this.connections.concat([
							dojo.connect(this.store, "onNew", this, "_onNew"),
							dojo.connect(this.store, "onSet", this, "_onSet"),
							dojo.connect(this.store, "onDelete", this, "_onDelete")
						]);
					}
				},
				destroy: function() {
				        // summary: destroys this object, releasing connections to the store
				        dojo.forEach(this.connections, dojo.disconnect);
				},
				reset: function() {
					// summary: Resets the root of this model to null.
					this.root = null;
				},
				getRoot: function(onItem, onError){
					// summary:
				    //            Calls onItem with the root item for the tree, possibly a fabricated item.
				    //			  Any request for data that is sent to the server should also include
				    //			  the request parameters in addQueryParams. 
					//            Throws exception on error.
					this.root = this.store.rootNode;
					onItem(this.root);
				},
				mayHaveChildren: function(/*Object*/ item){
				        // summary:
				        //            Tells if an item has or may have children.  Implementing logic here
				        //            avoids showing +/- expando icon for nodes that we know don't have children.
				        //            (For efficiency reasons we may not want to check if an element actually
				        //            has children until user clicks the expando node)
				        return this.store.getValue(item, "hasChildren");
				
				},
				getChildren: function(/*Object*/ parentItem, /*function(items)*/ onComplete, /*function*/ onError){
				        // summary:
				        // 			  Calls onComplete() with array of child items of given parent item, all loaded.
				        //			  Any request for data that is sent to the server should also include
				    	//			  the request parameters in addQueryParams. 
				        //            Throws exception on error.
						var store = this.store;		
						if(this.mayHaveChildren(parentItem)){
							parentItem._children = this.store.getValues(parentItem, "children");
				        	if(parentItem._children && parentItem._children.length > 0){
				        		onComplete(parentItem._children);
				        	}else{
				        		//does not have children so return empty array
				        		parentItem._children = null;
				        		onComplete(parentItem._children);
				        	}
				        }else {
				        	// does not have children so return empty array
				        	parentItem._children = null;
				        	onComplete(parentItem._children);
				        }        
				},
				// =======================================================================
				// Inspecting items
				getIdentity: function(/* item */ item){
				        // summary: returns identity for an item
				        return this.store.getIdentity(item);
				},
				getLabel: function(/*dojo.data.Item*/ item){
				        // summary: get the label for an item
				        return this.store.getLabel(item);
				},
				getParent: function(/*dojo.data.Item*/ item) {
					// summary: returns the parent item of this item argument
					return this.store.getValue(item, "parent");
				},
				// =======================================================================
				// Write interface
				newItem: function(/* Object? */ args, /*Item?*/ parent){
					// summary
			        // Creates a new item.   See dojo.data.api.Write for details on args.
					if(parent.getRootNavigation)
					{
						parent = com.ibm.mm.builder.utils.navigationUtil.getNavigationRootNode(parent.getID());
					}
			        var newPage = this.store.newItem(args, {parent: parent});
					var es = com.ibm.mashups.iwidget.services.ServiceManager.getService("eventService");
					if (args.openPageId){
						this.store.save({item: newPage, onComplete: args.onComplete, onError: args.onError, scope: args.scope, openPageId:args.openPageId, pageLoadedSpaceId:args.pageLoadedSpaceId});
						// to do the real broadcast event thing, broadcast the event and open the global page
						es.broadcastEvent("com.ibm.mashups.builder.saveCheck", dojo.hitch(this,function(){
							es.broadcastEvents(args.eventsArray || [],args.lastVisitedPage,args.openPageId);
						}));
					}else{
						this.store.save({item: newPage, onComplete: args.onComplete, onError: args.onError, scope: args.scope});
						es.broadcastEvent("com.ibm.mashups.builder.saveCheck", dojo.hitch(this,function(){
							es.broadcastEvent("com.ibm.mashups.builder.switchPage", this.getIdentity(newPage));
						})); 						
					}
			        return newPage;
				},
				deleteItem: function(/*Object*/args, /*item*/ item){
					// This is a fake delete item method, which only will remove global page.
					var parent = this.getParent(item);
					// key to set the global page as null, so the parent and children info will back to normal
					this.store.globalPage = null;
				    this._removeChildItem(parent, item, null, dojo.hitch(this, function(){
				    	this.onTreeChange(parent, null, parent);
				    }));
					var es = com.ibm.mashups.iwidget.services.ServiceManager.getService("eventService");
					if (args.openPagePayload){
						// do not switch to the last visited non-global page, we will open another global page instead.		
					}else{
						// go to last visited non-global page
						es.broadcastEvent("com.ibm.mashups.builder.saveCheck", dojo.hitch(this,function(){
							es.broadcastEvent("com.ibm.mashups.builder.switchPage", args.lastVisitedPage);
						})); 						
					}	
				},	
				clearGlobalPageItem:function(/*Object*/args, /*item*/ item){
					var parent = this.getParent(item);
					this.store.globalPage = null;
				    this._removeChildItem(parent, item, null, dojo.hitch(this, function(){
				    	this.onTreeChange(parent, null, parent);
				    }));
				},			
				isCreated: function(/*item*/ item) {
					// summary: Indicates whether or not the item has been created on the server and
					//		the server-generated values were properly applied to the client-side
					//		object.	NOTE: Any client-side modifications	to the values of an item for 
					// 		which isCreated returns true can potentially modify the server-generated 
					// 		values to be out-of-sync with the server.
					// 		Therefore, it's important to note that isCreated merely indicates whether
					//		or not the client-side item has a server-side resource associated with it,
					// 		and any values that the server generated for the resource (such as a unique
					return this.store.isCreated(item);	
				},
				addOnCreate: function(/*item*/ item, /*Function*/ callback) {
					// summary: Register callback to be called when the given item is created on the server.
					//		If the item is already created, this will call the callback immediately.
					return this.store.addOnCreate(item, callback);			
				},
				pasteItem: function(/*Item*/ childItem, /*Item*/ oldParentItem, /*Item*/ newParentItem, /*Boolean*/ copy, /*Item*/ nextSibling, /*Boolean?*/ extra){
				        // summary:
				        //            Move an item from one parent item to another or to another position under the same parent.
				        //            Used in drag & drop.
				        // See dojo.data.api.Write for details on args.

				        this.store.setValue(childItem, "position", {newParent: newParentItem, 
				        											oldParent: oldParentItem, 
				        											next: nextSibling});	        	
				        this.store._isDirty = true;
				        this.store.save({});
				        this.store.onSaveComplete();
				},
				toString: function() {
					return "[TreeModel " + this.declaredClass + "]";
				},
				_onNew: function(item, parentInfo){
					this.newPageItem = item;
					this.newPageParent = parentInfo.parent;
				    this._insertChildItem(this.newPageParent, this.newPageItem, null, dojo.hitch(this, function(){
				    	this.onTreeChange(this.newPageItem, null, this.newPageParent);
				    }));
				},
				_onDelete: function(item){
									
				},				
				_onSet: function(/* Item */ item, /*String*/ attribute, /*Object | Array*/ oldValue, /*Object | Array*/ newValue){				
					
					if(attribute.indexOf("parent") > -1) {
						var newParent = null;
						if(attribute == "parent_next"){
							newParent = this.store.getValue(item, "parent");
						}else{
							newParent = newValue;
							newValue = null;
						}

						this._removeChildItem(oldValue, item);
		        		this._insertChildItem(newParent, item, newValue, dojo.hitch(this, function(){
		        			this.onTreeChange(item, oldValue, newParent, newValue);
		        		}));		        		
					}else if(attribute == "position"){
						// if we're not reparenting, then we can only be moving
		        		this._moveChildItem(oldValue, item, newValue, dojo.hitch(this, function(){
		        			this.onTreeChange(item, oldValue, oldValue, newValue);
		        		}));
		        	}else {
						this.onChange(item);
					}
				},
				refreshItemChildren: function(/*Item*/ item, /*Function?*/ modifier) {
					item._children = null;					
					var func = modifier ? modifier : function(items) {
							this.onChildrenChange(item, items);
						};
					this.getChildren(item, dojo.hitch(this, func));
				},
				_insertChildItem: function(/*Item*/parentItem, /*Item*/ item, /*Item*/ newNextSibling, /*Function*/callbackfn) {
					this.getChildren(parentItem, dojo.hitch(this, function(items) {
						// refresh the item children with a function that will add the item first
						if(!items){
							items = [];
						}
						if(!newNextSibling) {
							items.push(item);
						} else {
							var id = this.getIdentity(newNextSibling);
							for(var i = 0; i < items.length; i++) {
								if(this.getIdentity(items[i]) == id) {
									items.splice(i, 0, item);
									break;
								}
							}
						}
						if(callbackfn) callbackfn();
						this.onChildrenChange(parentItem, items);
					}));
				},
				_removeChildItem: function(/*Item*/parentItem, /*Item*/ item, /*Function*/callbackfn) {
					this.getChildren(parentItem, dojo.hitch(this, function(items) {
						var id = this.store.getIdentity(item);
						if(items){
							for(var i = 0; i < items.length; i++) {
								if(this.store.getIdentity(items[i]) == id) {
									items.splice(i,1);	// remove the item from the array
									break;	// break out of the loop since we already found ourselves
								}
							}
						}
						if(callbackfn) callbackfn();
						this.onChildrenChange(parentItem, items);
					}));
				},
				_moveChildItem: function(/*Item*/parentItem, /*Item*/ item, /*String*/ newNextSibling, /*Function*/callbackfn) {
					this.getChildren(parentItem, dojo.hitch(this, function(items) {
						var id = this.store.getIdentity(item);
						var nextIndex = items.length - 1;
						for(var i = 0; i < items.length; i++) {
							if(this.store.getIdentity(items[i]) == id) {
								items.splice(i, 1);	// remove item from index i
								i--;
							}
							else if(newNextSibling && this.store.getIdentity(items[i]) == newNextSibling) {
								// insert item at index i before the existing item at that index without deleting it
								items.splice(i, 0, item);
								i++;
							}
						}
						if(!newNextSibling) {
							// if this is null, append the item to the end of the list
							items.push(item);
						}
						if(callbackfn) callbackfn();
						this.onChildrenChange(parentItem, items);
					}));
				},
				onTreeChange: function(/*Item*/ childItem, /*Item*/ oldParentItem, /*Item*/ newParentItem, /*Item*/ nextSiblingItem) {
					
				},
				onChange: function(/*Item*/ item){
					
				},
				onChildrenChange: function(/*Item*/ parentItem, /*Array*/ items){
					
				}
			}
);
