dojo.declare("com.ibm.mm.builder.systemwidget.modeToggle", [com.ibm.pb.iwidget.iWidgetBase], {
	toolbar: null,						// a reference to the toolbar
	openPull: null, 					// a reference to the open toolbar pull tab
	minPull: null, 						// a reference to the minimize minimize pull tab
	closePull: null, 					// a reference to the close toolbar pull tab
	mode: null,							// the toolbar mode, either open, closed or min
	pageMode: "view", 					// either view or edit, this is a builder/enabler API
	_keyPrefix: "pb.edit.mode.on.", 	// the prefix for the client side storage key to keep track of the toolbar mode, either open, closed or min
	_key: "",							// the client side storage key to keep track of the toolbar mode, either open, closed or min
	_toolbarDijitId: "customizeShelfContainer",		// the id of the toolbar iwidget/dijit
	onLoad: function() {
		// load this widget, ONLY if the user can edit the current page
		if(this.canEdit()) {
			// make the pull tabs visible
			var box = dojo.byId("theme-pullTab");
			dojo.removeClass(box, "lotusHidden");
			
			// find the open pull and connect the open function to it onclick
			this.bundle = dojo.i18n.getLocalization("com.ibm.bundles", "Shelf");
			this.openPull = dojo.byId("theme-open-pull");
			this.openPull.title = dojo.byId("theme-openTabAlt").innerHTML = this.bundle["utb_open_toolbar"];
			this._connect = dojo.connect(this.openPull, "onclick", this, "open");
			
			// find the close pull and connect the minimize function to it onclick
			this.minPull = dojo.byId("theme-min-pull");
			this.minPull.title = dojo.byId("theme-minTabAlt").innerHTML = this.bundle["utb_show_min_toolbar"];
			this._connect = dojo.connect(this.minPull, "onclick", this, "minimize");
			
			// find the close pull and connect the close function to it onclick
			this.closePull = dojo.byId("theme-close-pull");
			this.closePull.title = dojo.byId("theme-closeTabAlt").innerHTML = this.bundle["utb_close_toolbar"];
			this.minAlt = dojo.byId("theme-closeTabAlt");
			this._connect = dojo.connect(this.closePull, "onclick", this, "close");
			
			// find the rendering controller for the purpose of going in and out of edit mode
			this.controller = dojo.getObject("ibmCfg.controllers.rendering");
			
			// create the client side toolbar mode storage id based on the current user id, so the mode status is scoped by user
			var um = com.ibm.mashups.enabler.model.Factory.getUserModel();
			var userID = um.findCurrentUser().start().getID(); 
			this._key = this._keyPrefix+userID;
			
			// find and execute the stored state of the bar
			var state = this.getStoredState();
			if(state === "open") this.open();
			else if(state === "min") this.minimize();
			else if(state === "closed") this.close();
		}
	},
	minimize: function() {
		// minimize the toolbar so only the statusbar is visible
		if(this.toolbar) dojo.addClass(this.toolbar, "lotusHidden");
		
		// ignore if the toolbar is already in the minimized state
		if(this.mode === "min") return;
		
		// disable the minimize tab pull and enable the others
		dojo.addClass(this.minPull, "utb-disabled");
		dojo.removeClass(this.openPull, "utb-disabled");
		dojo.removeClass(this.closePull, "utb-disabled");
		
		// mode: open => min
		if(this.mode === "open") {
			// change the toolbar mode to min
			this.setStoredState("min");
		// mode: closed => min
		} else {
			// go to page edit mode
			this.setStoredState("min");
			this.pageMode = "edit";
			if(this.controller) this.controller.changePageMode(this.pageMode);
			else this._broadcastModeChange(this.pageMode);
		}
	},
	open: function() {
		// open the toolbar all the way
		// ignore if the toolbar is already in open mode
		if(this.mode === "open") return;
		
		// disable the open tab pull and enable the others
		dojo.addClass(this.openPull, "utb-disabled");
		dojo.removeClass(this.minPull, "utb-disabled");
		dojo.removeClass(this.closePull, "utb-disabled");
		
		// change the text of the minimize toolbar control
		this.minPull.title = dojo.byId("theme-minTabAlt").innerHTML = this.bundle["utb_min_toolbar"];
		
		// make the toolbar visible
		if(this.toolbar) dojo.removeClass(this.toolbar, "lotusHidden");
		
		// mode: min => open
		if(this.mode === "min") {
			// change the toolbar mode to open
			this.setStoredState("open");
		// mode: closed => open
		} else {
			// go into edit mode
			this.setStoredState("open");
			this.pageMode = "edit";
			if(this.controller) this.controller.changePageMode(this.pageMode);
			else this._broadcastModeChange(this.pageMode);
		}
	},
	close: function() {
		// close the toolbar all the way
		// ignore if the toolbar is already in closed mode
		if(this.mode === "closed") return;
		
		// disable the close tab pull and enable the others
		dojo.addClass(this.closePull, "utb-disabled");
		dojo.removeClass(this.openPull, "utb-disabled");
		dojo.removeClass(this.minPull, "utb-disabled");

		// change the text of the minimize toolbar control
		this.minPull.title = dojo.byId("theme-minTabAlt").innerHTML = this.bundle["utb_show_min_toolbar"];			
			
		// go to view mode
		this.setStoredState("closed");
		this.pageMode = "view";
		if(this.controller) this.controller.changePageMode(this.pageMode);
		else this._broadcastModeChange(this.pageMode);
	},
	getStoredState: function() {
		// get the stored state of the toolbar: open, closed or min
		if(typeof localStorage != "undefined" && localStorage[this._key]) return localStorage[this._key];
		else return "closed";
	},
	setStoredState: function(state) {
		// set the stored state of the toolbar: open, closed or min
		this.mode = state;
		if(typeof localStorage !== "undefined") {
			// all supported browsers but IE7
			localStorage.removeItem(this._key);
			localStorage[this._key] = state;
		}
	},
	onPageModeChanged: function(iEvent) {
		// triggered by the page change mode iEvent
		var mode = iEvent.payload;
		if(mode === "edit") {
			dojo.addClass(dojo.body(), "edit-mode");
			if(!this.toolbar) {
				// find the toolbar iWidget/dijit
				var widget = dijit.byId(this._toolbarDijitId);
				if(widget) {
					this.toolbar = widget.containerDivs;
					dojo.connect(widget, "onShowTab", this, "open");
				}
			}
			// minimize or open the toolbar on edit
			if(this.mode === "min") this.minimize();
			else this.open();
		} else {
			// close the toolbar on view mode
			dojo.removeClass(dojo.body(), "edit-mode");
			this.close();
		}
	},
	canEdit: function() {
		// returns true if the current user can edit the current page in any way
		var navModel = com.ibm.mashups.enabler.navigation.Factory.getNavigationModel();
		var pageId = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getID();
		var page = navModel.find(pageId).start();
		if(page.hasRole(com.ibm.mashups.enabler.ac.RoleType.ADMIN) || 
			page.hasRole(com.ibm.mashups.enabler.ac.RoleType.EDITOR) ||
			page.hasRole("Privileged User")) {
			return true;
		} else return false;
	},
	_broadcastModeChange: function(mode) {
		// broadcasts the changePageMode ievent with a mode payload
		var currentMode = com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().getPageMode();
		var userModel = com.ibm.mashups.enabler.user.Factory.getUserModel();
		if (userModel.getAnonymousMode() != com.ibm.mashups.enabler.user.AnonymousMode.ANONYMOUS && currentMode != mode) {
			if(dojo.isSafari || dojo.isOpera){
				var me = this;
				setTimeout(function(){me.evtSvc.broadcastEvent("com.ibm.mashups.builder.changePageMode", mode);},500);
			} else {
				this.evtSvc.broadcastEvent("com.ibm.mashups.builder.changePageMode", mode);
			}
			com.ibm.mashups.builder.model.Factory.getRuntimeModel().getCurrentPage().setPageMode(mode);
		}
	},
	onUnload: function() {
		dojo.disconnect(this._connect);
	}
});