/* Copyright IBM Corp. 2013  All Rights Reserved.                    */
dojo.provide("com.ibm.mm.builder.spaceCopyWidget");
dojo.require("dojo.string");
dojo.require("com.ibm.mm.builder.iwidget.spacemanager.SpaceManagerConstants");
dojo.declare("com.ibm.mm.builder.spaceCopyWidget", null, {	
	
	eventService: com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.iwidget.services.EventService.SERVICE_NAME),
	
	onLoad: function(){

		this.widgetId = this.iContext.widgetId;
		this.WID = "_"+ this.widgetId + "_";
		this.cs=com.ibm.mashups.services.ServiceManager.getService(com.ibm.mashups.enabler.services.ConfigService.SERVICE_NAME);
		
		this.resourceBundle = com.ibm.mm.builder.coreWidgetsStrings;	
		this.resourceMessage = com.ibm.mm.builder.coreWidgetsMessages;				
	},
	
	hide: function(){
		this.eventService.broadcastEvent("com.ibm.mashups.builder.closeMashupDialog", this.widgetId);
	},
	
	onView: function(){		
		dojo.parser.parse(this.iContext.getRootElement());
		dojo.byId(this.WID+"space_name_label").innerHTML = this.resourceBundle.COPY_SPACE_NAME;
		dojo.byId(this.WID+"space_desc_label").innerHTML = this.resourceBundle.COPY_SPACE_DESC;
		
		var serverType = this.cs.getValue(com.ibm.mashups.enabler.services.ConfigConstants.SERVER_TYPE);
		if (serverType==com.ibm.mashups.enabler.services.ConfigConstants.SERVER_TYPE_PORTAL) {
			dojo.style(dojo.byId(this.WID+"copy_space_hint"), "display", "none");
		}
	},
	
	_copySpace: function(clonedSpaceId, newTitle, newDescription){
		
		var spaceUtil = com.ibm.mm.builder.utils.spaceUtil;
		var spaceModel = spaceUtil.getSpaceModel();
		
		var clonedSpace = spaceUtil.getSpaceNode(clonedSpaceId);
		var isFav = clonedSpace.isFavorite();
		
		var space = spaceModel.create({ "clonedSpace" : clonedSpaceId});

		space.setTitle(newTitle, builderConfig.locale);
		space.setDescription(newDescription, builderConfig.locale);
		if(isFav){
 		    space.setFavorite(true);
		}
					
		spaceModel.insert(space);
		var deferred = spaceModel.commit();
		
        var newSpaceID = null;
        deferred.setOperationCallback(dojo.hitch(this, function(newSpace){
            newSpaceID = newSpace.getID();
        }));
        deferred.setFinishedCallback(dojo.hitch(this, function(statusCode){
            if (newSpaceID !== null) {
                this.eventService.broadcastEvent("com.ibm.mashups.builder.spacemanager.spaceCreated", newSpaceID);
            
                if (!this.copySpaceInSpaceManager) {
                    this.eventService.broadcastEvent("com.ibm.mashups.builder.switchSpace", newSpaceID);
                }
                spaceUtil.getSpaceModel(true);
            }
        }));
		
		deferred.start();
	},
	
	_getTitle: function(){
		return dojo.byId(this.WID+"SpaceName").value;
	},
	
	_getDescription: function(){
		return dojo.byId(this.WID+"SpaceDesc").value;
	},	
	
	_checkTitle: function(title) {
		title = dojo.trim(title || "");		
		
		if (!title){
			return {res:false, errorMsg: this.resourceMessage.E_SPACE_MANAGER_SPACE_NAME_IS_REQUIRED_0};
		} 		
		if (title.length > 256){
			return {res:false, errorMsg: this.resourceMessage.E_SPACE_MANAGER_SPACE_NAME_HAS_MAX_LIMIT_0};
		} 		
		var match = ibmConfig[com.ibm.mashups.enabler.services.ConfigConstants.BUILDER_TITLE_FILTER_REGEXP];		
		if (title.match(match)) {
			return {
				res: false,
				errorMsg: this.resourceMessage.E_SPACE_MANAGER_SPACE_NAME_HAS_INVALID_CHAR_0
			};
		}	
		
		return {res:true};		
	},
	
	
	_verifyTitle : function(title){
		var spaceNameInput = dojo.byId(this.WID+"SpaceName"); 
		
		var checkResult = this._checkTitle(title);       

		if (!checkResult.res) {
			this._setAsAriaInvalid(spaceNameInput);
			
			this.messageControl.attr("message", checkResult.errorMsg);			
			this.messageControl.show();			
        	return false;
		}
	    		
		this._removeAriaInvalid(spaceNameInput);
	    return true;
	},
	
	_verifyDescription : function(description){
		
		if (!dojo.isString(description)) {
			return false;
		}
		
		var spaceDescInput = dojo.byId(this.WID+"SpaceDesc");
	    if (description.length >= 512) {
			this._setAsAriaInvalid(spaceDescInput);
	        this.messageControl.attr("message", this.resourceMessage.E_SPACE_MANAGER_SPACE_DESC_HAS_MAX_LIMIT_0);			
			this.messageControl.show();
	        return false;
	    }
		
		this._removeAriaInvalid(spaceDescInput);
		return true;
	},
	
	_setAsAriaInvalid: function(domNode){
		dojo.attr(domNode, "aria-invalid", "true");
	},
	
	_removeAriaInvalid: function(domNode){
		dojo.removeAttr(domNode, "aria-invalid");
	},
	
	_verifyInput: function(title, description){
		
		return this._verifyTitle(title) && this._verifyDescription(description);
	},
	
	save: function(){	
		
		var clonedSpaceId = this.spaceId;
		
		//validation
		var title = this._getTitle();
		var description = this._getDescription();
		
		if (!this.messageControl) {
			dojo.registerModulePath("com.ibm.lotus.oneui.v100.messages", this.iContext.io.rewriteURI("../dijit-library/oneUI.versionized/messages"));
	    	dojo.require("com.ibm.lotus.oneui.v100.messages.Message");
			this.messageControl = new com.ibm.lotus.oneui.v100.messages.Message({
				messageType: 'error'
			}, this.WID + "messageBox");
			this.messageControl.hide();		
		}
		//validation
		if(!this._verifyInput(title, description)){
			return;
		}			
			
		dojo.attr(this.WID + "btn_save","disabled",true);
		
		this._copySpace(clonedSpaceId, title, description);
			
		this.messageControl.hide();			
		this.hide();
	},
	
	onCopySpace: function(iEvent){
		if(iEvent.payload && iEvent.payload.spaceId){
			this.spaceId = iEvent.payload.spaceId;
			this.copySpaceInSpaceManager = iEvent.payload.copySpaceInSpaceManager;
		}else{
			this.spaceId = iEvent.payload;
			this.copySpaceInSpaceManager = false;
		}
		this.cleanUI();
		
	},
	
	cleanUI: function(){
		if (this.messageControl) {
			this.messageControl.hide();
		}
		dojo.attr(this.WID + "btn_save","disabled",false);
		
		var spaceUtil = com.ibm.mm.builder.utils.spaceUtil;
		var spaceModel = spaceUtil.getSpaceModel();
		var space = spaceModel.find(this.spaceId).start();
		
        if (space) {
            
			var title = space.getTitle(builderConfig.locale);
			if (!title) {
				title = space.getTitle(this.cs.getValue("default.locale"));
			}
				
			var description = space.getDescription(builderConfig.locale);
			if (!description) {
				description = space.getDescription(this.cs.getValue("default.locale"));
			}

			dojo.byId(this.WID+"SpaceName").value = dojo.string.substitute(this.resourceBundle.COPY_SPACE_COPY_OF,[title]);
			
			dojo.byId(this.WID+"SpaceDesc").value = description ? description : "";
			
        } else {
			//FIXME: ADD error message.
            //alert("Space with id '" + spaceId + "' not found.");
            return;
        }
	}
});
